package com.googlecode.mycontainer.commons.rhino;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Serializable;
import java.net.URL;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ContextFactory;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.tools.shell.ShellContextFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.googlecode.mycontainer.commons.io.IOUtil;

public class RhinoBox implements Serializable {

	private static final long serialVersionUID = -2958495434782407449L;

	private static final Logger LOG = LoggerFactory.getLogger(RhinoBox.class);

	private static final ShellContextFactory CONTEXT_FACTORY = new RhinoContextFactory();

	static {
		ContextFactory.initGlobal(CONTEXT_FACTORY);
	}

	private RhinoScope scope;

	public RhinoBox(ScriptableObject scope) {
		enter();
		try {
			if (scope == null) {
				scope = getContext().initStandardObjects();
			}

			this.scope = new RhinoScope(scope);
			load("rhinobox/server/rhinobox.js");
		} finally {
			exit();
		}
	}

	public Object load(String resource) {
		return load(getClass().getClassLoader().getResource(resource));
	}

	public Object load(URL resource) {
		Object ret = source(resource);
		if (ret instanceof Function) {
			ret = invoke(getClass(), (Function) ret, this);
		}
		return ret;
	}

	public RhinoBox() {
		this(null);
	}

	public static ShellContextFactory getContextFactory() {
		return CONTEXT_FACTORY;
	}

	public Context getContext() {
		Context ret = Context.getCurrentContext();
		if (ret == null) {
			throw new RuntimeException("no context");
		}
		return ret;
	}

	public RhinoScope getScope() {
		return scope;

	}

	public Object source(String resource) {
		return source(getClass().getClassLoader().getResource(resource));
	}

	public Object source(URL resource) {
		InputStream in = null;
		try {
			String name = resource.toString();
			LOG.info("Source: " + resource);
			in = resource.openStream();
			return source(in, name);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			IOUtil.close(in);
		}
	}

	public Object source(InputStream in, String resource) {
		Reader reader = new InputStreamReader(in);
		return source(reader, resource);
	}

	public Object source(Reader reader, String resource) {
		try {
			return getContext().evaluateReader(scope.getScope(), reader,
					resource, 1, null);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void enter() {
		Context.enter();
	}

	public static void exit(RhinoBox interpreter) {
		if (interpreter != null) {
			interpreter.exit();
		}
	}

	public void exit() {
		Context.exit();
	}

	public Object eval(String code, Class<?> name) {
		return eval(code, "rhino." + name.getSimpleName());
	}

	public Object eval(String code, String name) {
		return getContext().evaluateString(scope.getScope(), code, name, 1,
				null);
	}

	public Object invoke(Class<?> name, String function, Object... values) {
		return invokeScope(name, function, (Scriptable) null, values);
	}

	public Object invokeScope(Class<?> name, String function, String scope,
			Object... values) {
		Scriptable s = this.scope.getScope();
		if (scope != null) {
			s = (ScriptableObject) eval(scope, name);
		}
		return invokeScope(name, function, s, values);
	}

	public Object invokeScope(Class<?> name, String function, Scriptable scope,
			Object... values) {
		Scriptable s = scope;
		if (s == null) {
			s = this.scope.getScope();
		}
		Function f = (Function) eval(function, name);
		return f.call(getContext(), this.scope.getScope(), s, values);
	}

	public Object invoke(Class<?> name, Function function, Object... values) {
		return function.call(getContext(), scope.getScope(), scope.getScope(),
				values);
	}

	public Logger getLogger() {
		return LoggerFactory.getLogger("RhinoBox");
	}

	public static void main(String[] args) {
		RhinoBox box = DefaultRhinoBoxBuilder.getBox();
		org.mozilla.javascript.tools.shell.Main.exec(args);
	}
}
